"""
Utility functions for interfacing with the Django messages framework.
"""

from django.conf import settings
from django.contrib import messages
from django.utils.translation import ugettext as _

from enterprise.utils import get_configuration_value


def add_consent_declined_message(request, enterprise_customer, item):
    """
    Add a message to the Django messages store indicating that the user has declined data sharing consent.

    Arguments:
        request (HttpRequest): The current request.
        enterprise_customer (EnterpriseCustomer): The EnterpriseCustomer associated with this request.
        item (str): A string containing information about the item for which consent was declined.
    """
    messages.warning(
        request,
        _(
            '{strong_start}We could not enroll you in {em_start}{item}{em_end}.{strong_end} '
            '{span_start}If you have questions or concerns about sharing your data, please contact your learning '
            'manager at {enterprise_customer_name}, or contact {link_start}{platform_name} support{link_end}.{span_end}'
        ).format(
            item=item,
            em_start='<em>',
            em_end='</em>',
            enterprise_customer_name=enterprise_customer.name,
            link_start='<a href="{support_link}" target="_blank">'.format(
                support_link=get_configuration_value('ENTERPRISE_SUPPORT_URL', settings.ENTERPRISE_SUPPORT_URL),
            ),
            platform_name=get_configuration_value('PLATFORM_NAME', settings.PLATFORM_NAME),
            link_end='</a>',
            span_start='<span>',
            span_end='</span>',
            strong_start='<strong>',
            strong_end='</strong>',
        )
    )


def add_missing_price_information_message(request, item):
    """
    Add a message to the Django messages store indicating that we failed to retrieve price information about an item.

    :param request: The current request.
    :param item: The item for which price information is missing. Example: a program title, or a course.
    """
    messages.warning(
        request,
        _(
            '{strong_start}We could not gather price information for {em_start}{item}{em_end}.{strong_end} '
            '{span_start}If you continue to have these issues, please contact '
            '{link_start}{platform_name} support{link_end}.{span_end}'
        ).format(
            item=item,
            em_start='<em>',
            em_end='</em>',
            link_start='<a href="{support_link}" target="_blank">'.format(
                support_link=get_configuration_value('ENTERPRISE_SUPPORT_URL', settings.ENTERPRISE_SUPPORT_URL),
            ),
            platform_name=get_configuration_value('PLATFORM_NAME', settings.PLATFORM_NAME),
            link_end='</a>',
            span_start='<span>',
            span_end='</span>',
            strong_start='<strong>',
            strong_end='</strong>',
        )
    )


def add_unenrollable_item_message(request, item):
    """
    Add a message to the Django message store indicating that the item (i.e. course run, program) is unenrollable.

    :param request: The current request.
    :param item: The item that is unenrollable (i.e. a course run).
    """
    messages.info(
        request,
        _(
            '{strong_start}Something happened.{strong_end} '
            '{span_start}This {item} is not currently open to new learners. Please start over and select a different '
            '{item}.{span_end}'
        ).format(
            item=item,
            strong_start='<strong>',
            strong_end='</strong>',
            span_start='<span>',
            span_end='</span>',
        )
    )


def add_generic_error_message_with_code(request, error_code):
    """
    Add message to request indicating that there was an issue processing request.

    Arguments:
        request: The current request.
        error_code: A string error code to be used to point devs to the spot in
                    the code where this error occurred.

    """
    messages.error(
        request,
        _(
            '{strong_start}Something happened.{strong_end} '
            '{span_start}Please reach out to your learning administrator with '
            'the following error code and they will be able to help you out.{span_end}'
            '{span_start}Error code: {error_code}{span_end}'
            '{span_start}Username: {username}{span_end}'
        ).format(
            error_code=error_code,
            username=request.user.username,
            strong_start='<strong>',
            strong_end='</strong>',
            span_start='<span>',
            span_end='</span>',
        )
    )



"""
Management command for updating enterprise user role assignments
with appropriate ``enterprise_customer`` and ``applies_to_all_contexts`` values.
"""

import logging
from collections import defaultdict

from django.contrib import auth
from django.core.management.base import BaseCommand

from enterprise import roles_api
from enterprise.config.models import UpdateRoleAssignmentsWithCustomersConfig
from enterprise.constants import (
    ENTERPRISE_ADMIN_ROLE,
    ENTERPRISE_LEARNER_ROLE,
    ENTERPRISE_OPERATOR_ROLE,
    SYSTEM_ENTERPRISE_CATALOG_ADMIN_ROLE,
)
from enterprise.models import EnterpriseCustomerUser, SystemWideEnterpriseUserRoleAssignment
from enterprise.utils import batch

log = logging.getLogger(__name__)
User = auth.get_user_model()


# pylint: disable=logging-fstring-interpolation
class Command(BaseCommand):
    """
    Management command for creating enterprise role assignments
    with a foreign key to an EnterpriseCustomer, or an explicit boolean
    flag indicating that the assignment grants the role to the user
    for every EnterpriseCustomer.
    """
    help = """
    Applies explicit enterprise customer context for enterprise admin, learner, and catalog system role assignments.
    It also sets `applies_to_all_contexts` to true for assignments of the `enterprise_openedx_operator` role.
    Example usage:
      # Do a dry run for admin assignments of some customer
      ./manage.py lms update_role_assignments_with_customers --role=enterprise_admin --dry-run --enterprise-customer-uuid=00000000-1111-2222-3333-444444444444
      # Do a real run for all operators
      ./manage.py lms update_role_assignments_with_customers --role=enterprise_openedx_operator
      # Process everything for everyone
      ./manage.py lms update_role_assignments_with_customers
    """

    def add_arguments(self, parser):
        """
        Entry point for subclassed commands to add custom arguments.
        """
        parser.add_argument(
            '--role',
            action='store',
            dest='role',
            default=None,
            help='Specifies which user role assignments to update.  If unspecified, will update for all roles.'
        )
        parser.add_argument(
            '--batch-size',
            action='store',
            dest='batch_size',
            default=500,
            help='Number of user role asssignments to update in each batch of updates.',
            type=int,
        )
        parser.add_argument(
            '--enterprise-customer-uuid',
            action='store',
            dest='enterprise_customer_uuid',
            help='The enterprise customer to limit role assignments to.',
        )
        parser.add_argument(
            '--dry-run',
            action='store_true',
            dest='dry_run',
            help=(
                'If set, no updates or creates will occur; will instead iterate over '
                'the assignments that would be modified or created'
            ),
        )
        parser.add_argument(
            '--undo',
            action='store_true',
            dest='undo',
        )
        parser.add_argument(
            '--args-from-database',
            action='store_true',
            dest='args_from_database',
            help='If true, read arguments from a DB config model.',
        )

    def handle(self, *args, **options):
        """
        Entry point for managment command execution.
        """
        valid_roles = (
            ENTERPRISE_ADMIN_ROLE,
            ENTERPRISE_LEARNER_ROLE,
            ENTERPRISE_OPERATOR_ROLE,
            SYSTEM_ENTERPRISE_CATALOG_ADMIN_ROLE,
        )

        if options['args_from_database']:
            options.update(self._latest_settings_from_db())

        if not options['role']:
            for role in valid_roles:
                self._handle(
                    options['batch_size'],
                    role,
                    enterprise_customer_uuid=options.get('enterprise_customer_uuid', None),
                    dry_run=options['dry_run']
                )
        elif options['role'] in valid_roles:
            self._handle(
                options['batch_size'],
                options['role'],
                enterprise_customer_uuid=options.get('enterprise_customer_uuid', None),
                dry_run=options['dry_run'],
            )
        else:
            log.error(f"{options['role']} is not a valid role.")
            return

    def _handle(self, batch_size, role_name, enterprise_customer_uuid=None, dry_run=False):
        """
        Does all of the actual work on assignments of a given role.
        """
        if role_name == ENTERPRISE_OPERATOR_ROLE:
            self._handle_operators(dry_run)
            return

        ent_customer_user_ids = self._get_all_enterprise_customer_user_ids(enterprise_customer_uuid)

        log.info(f'There are {len(ent_customer_user_ids)} total enterprise customer user ids to process.')

        for ecu_ids_batch in batch(ent_customer_user_ids, batch_size=batch_size):
            log.info(f'Processing batch of ECU ids between {ecu_ids_batch[0]} and {ecu_ids_batch[-1]}.')

            customers_by_user_id = self._get_customers_by_user_id(ecu_ids_batch)

            assignments_by_user_id_role = self._get_role_assignments_by_user_id_and_role(
                customers_by_user_id.keys(),
                role_name,
            )
            log.info(
                f'There are {len(assignments_by_user_id_role)} assignments '
                f'of the {role_name} role to process in this batch'
            )
            self._handle_non_operators(
                customers_by_user_id,
                assignments_by_user_id_role,
                dry_run,
            )

    def _handle_operators(self, dry_run):
        """
        If not a dry run, modifies all enterprise_openedx_operator role assignments
        to have `applies_to_all_contexts` set to True (because it's implied that a user with the operator role
        has that role for all contexts/customers).

        If `dry_run` is true, we don't perform a database update; instead, a message will be logged
        indicating how many assignments would be updated.
        """
        updated_assignments = []
        queryset = SystemWideEnterpriseUserRoleAssignment.objects.filter(role__name=ENTERPRISE_OPERATOR_ROLE)
        for assignment in queryset:
            assignment.applies_to_all_contexts = True
            updated_assignments.append(assignment)

        log.info(f'There are {len(updated_assignments)} operator role assignments to update.')
        if not dry_run:
            SystemWideEnterpriseUserRoleAssignment.objects.bulk_update(
                updated_assignments,
                ['applies_to_all_contexts'],
                batch_size=100,
            )
            log.info('All operator role assignments have been updated.')

    def _handle_non_operators(self, customers_by_user_id, assignments_by_user_id_role, dry_run):
        """
        For a given mapping of users to enterprise customers, and (user, role) to assignments,
        this method creates one SystemWideEnterpriseUserRoleAssignment per user-customer link,
        with a role based on the `assignments_by_user_id_role` mapping.
        """
        # we'll bulk create the new objects in the DB after we loop through the assignment-user mapping
        assignments_to_create = []

        roles_by_name = roles_api.roles_by_name()

        for (user_id, role_name), role_assignment_set in assignments_by_user_id_role.items():
            linked_customers_by_uuid = {
                customer.uuid: customer for customer in customers_by_user_id[user_id]
            }

            # Some users may already have a role explicitly associated with an enterprise customer,
            # so we first compute the difference between explicit assignments and all linked enterprises.
            existing_customer_assignments = {
                assignment.enterprise_customer.uuid for assignment in role_assignment_set
                if assignment.enterprise_customer
            }
            customer_uuids_to_assign = list(set(linked_customers_by_uuid.keys()) - existing_customer_assignments)
            if not customer_uuids_to_assign:
                continue

            for customer_uuid in customer_uuids_to_assign:
                assignments_to_create.append(
                    SystemWideEnterpriseUserRoleAssignment(
                        user_id=user_id,
                        role=roles_by_name[role_name],
                        enterprise_customer=linked_customers_by_uuid[customer_uuid],
                    )
                )

        log.info(
            f'There are {len(assignments_to_create)} assignments to create in this batch.'
        )

        if dry_run:
            log.info('This is a dry run, no updates or creates will happen for this batch.')
            return

        SystemWideEnterpriseUserRoleAssignment.objects.bulk_create(
            assignments_to_create, batch_size=100
        )

    def _get_all_enterprise_customer_user_ids(self, enterprise_customer_uuid=None):
        """
        Returns a list of all `EnterpriseCustomerUser` ids, or, if `enterprise_customer_uuid` is not null,
        a list of all such ids related to the customer with the given UUID.
        """
        ecu_queryset = EnterpriseCustomerUser.objects.all()
        if enterprise_customer_uuid:
            ecu_queryset = EnterpriseCustomerUser.objects.filter(enterprise_customer__uuid=enterprise_customer_uuid)

        return ecu_queryset.order_by('id').values_list('id', flat=True)

    def _get_customers_by_user_id(self, enterprise_customer_user_ids):
        """
        Returns a mapping of auth.User ids to a set of EnterpriseCustomers
        the user is linked to.
        """
        queryset = EnterpriseCustomerUser.objects.filter(
            id__in=enterprise_customer_user_ids
        )
        customers_by_user_id = defaultdict(set)
        for ecu in queryset:
            customers_by_user_id[ecu.user_id].add(ecu.enterprise_customer)

        return customers_by_user_id

    def _get_role_assignments_by_user_id_and_role(self, user_ids, role_name):
        """
        Returns a mapping of (auth.User id, role name) to a set of SystemWideEnterpriseUserRoleAssignment objects
        associated with that user and role.
        """
        role_assignment_kwargs = {
            'user_id__in': user_ids,
            'role__name': role_name,
            'applies_to_all_contexts': False,
        }

        assignments_by_user_id_role = defaultdict(set)
        for assignment in SystemWideEnterpriseUserRoleAssignment.objects.filter(**role_assignment_kwargs):
            assignments_by_user_id_role[
                (assignment.user_id, assignment.role.name)
            ].add(assignment)

        return assignments_by_user_id_role

    def _latest_settings_from_db(self):
        """
        Return the latest settings from the model-defined settings
        for this management command.
        """
        config = UpdateRoleAssignmentsWithCustomersConfig.current()
        return {
            'role': config.role,
            'batch_size': config.batch_size,
            'enterprise_customer_uuid': config.enterprise_customer_uuid,
            'dry_run': config.dry_run,
        }



"""
URL definitions for version 1 of the Consent API.

Currently supports the following services:
    ``data_sharing_consent``: Allows for getting, providing, and revoking consent to share data.
"""

from django.conf.urls import url

from .views import DataSharingConsentView

urlpatterns = [
    url(
        r'^data_sharing_consent$',
        DataSharingConsentView.as_view(),
        name='data_sharing_consent'
    ),
]



"""
Views for enterprise_learner_portal app.
"""

from edx_rest_framework_extensions.auth.jwt.authentication import JwtAuthentication
from rest_framework import permissions
from rest_framework.authentication import SessionAuthentication
from rest_framework.response import Response
from rest_framework.status import HTTP_400_BAD_REQUEST
from rest_framework.views import APIView

from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _

from enterprise.models import EnterpriseCourseEnrollment, EnterpriseCustomerUser
from enterprise.utils import NotConnectedToOpenEdX
from enterprise_learner_portal.api.v1.serializers import EnterpriseCourseEnrollmentSerializer

try:
    from openedx.core.djangoapps.content.course_overviews.api import get_course_overviews
except ImportError:
    get_course_overviews = None


class EnterpriseCourseEnrollmentView(APIView):
    """
    View for returning information around a user's enterprise course enrollments.
    """
    permission_classes = (permissions.IsAuthenticated,)
    authentication_classes = (JwtAuthentication, SessionAuthentication,)

    def get(self, request):
        """
        Returns a list of EnterpriseCourseEnrollment data related to the requesting user.

        Example response:

        [
            {
                "certificate_download_url": null,
                "course_run_id": "course-v1:edX+DemoX+Demo_Course",
                "course_run_status": "in_progress",
                "start_date": "2013-02-05T06:00:00Z",
                "end_date": null,
                "display_name": "edX Demonstration Course",
                "course_run_url": "http://localhost:18000/courses/course-v1:edX+DemoX+Demo_Course/course/",
                "due_dates": [],
                "pacing": "instructor",
                "org_name": "edX",
                "is_revoked": false,
                "is_enrollment_active": true
            }
        ]

        Query params:
          'enterprise_id' (UUID string, required): The enterprise customer UUID with which to filter
            EnterpriseCustomerRecords by.
          'is_active' (boolean string, optional): If provided, will filter the resulting list of enterprise
            enrollment records to only those for which the corresponding ``student.CourseEnrollment``
            record has an ``is_active`` equal to the provided boolean value ('true' or 'false').
        """
        if get_course_overviews is None:
            raise NotConnectedToOpenEdX(
                _('To use this endpoint, this package must be '
                  'installed in an Open edX environment.')
            )

        user = request.user
        enterprise_customer_id = request.query_params.get('enterprise_id', None)
        if not enterprise_customer_id:
            return Response(
                {'error': 'enterprise_id must be provided as a query parameter'},
                status=HTTP_400_BAD_REQUEST
            )

        enterprise_customer_user = get_object_or_404(
            EnterpriseCustomerUser,
            user_id=user.id,
            enterprise_customer__uuid=enterprise_customer_id,
        )
        enterprise_enrollments = EnterpriseCourseEnrollment.objects.filter(
            enterprise_customer_user=enterprise_customer_user
        )

        filtered_enterprise_enrollments = [record for record in enterprise_enrollments if record.course_enrollment]

        course_overviews = get_course_overviews([record.course_id for record in filtered_enterprise_enrollments])

        data = EnterpriseCourseEnrollmentSerializer(
            filtered_enterprise_enrollments,
            many=True,
            context={'request': request, 'course_overviews': course_overviews},
        ).data

        if request.query_params.get('is_active'):
            is_active_filter_value = None
            if request.query_params['is_active'].lower() == 'true':
                is_active_filter_value = True
            if request.query_params['is_active'].lower() == 'false':
                is_active_filter_value = False
            if is_active_filter_value is not None:
                data = [
                    record for record in data
                    if record['is_enrollment_active'] == is_active_filter_value
                ]

        return Response(data)

    def patch(self, request):
        """
        Patch method for the view.
        """
        if get_course_overviews is None:
            raise NotConnectedToOpenEdX(
                _('To use this endpoint, this package must be '
                  'installed in an Open edX environment.')
            )

        user = request.user
        enterprise_customer_id = request.query_params.get('enterprise_id', None)
        course_id = request.query_params.get('course_id', None)
        saved_for_later = request.query_params.get('saved_for_later', None)

        if not enterprise_customer_id or not course_id or saved_for_later is None:
            return Response(
                {'error': 'enterprise_id, course_id, and saved_for_later must be provided as query parameters'},
                status=HTTP_400_BAD_REQUEST
            )

        enterprise_customer_user = get_object_or_404(
            EnterpriseCustomerUser,
            user_id=user.id,
            enterprise_customer__uuid=enterprise_customer_id,
        )

        enterprise_enrollment = get_object_or_404(
            EnterpriseCourseEnrollment,
            enterprise_customer_user=enterprise_customer_user,
            course_id=course_id
        )

        # TODO: For now, this makes the change backward compatible, we will change this to true boolean support
        enterprise_enrollment.saved_for_later = saved_for_later.lower() == 'true'

        enterprise_enrollment.save()

        course_overviews = get_course_overviews([course_id])
        data = EnterpriseCourseEnrollmentSerializer(
            enterprise_enrollment,
            context={'request': request, 'course_overviews': course_overviews},
        ).data

        return Response(data)



"""
URL definitions for enterprise_learner_portal API endpoint.
"""

from django.conf.urls import url

from enterprise_learner_portal.api.v1.views import EnterpriseCourseEnrollmentView

urlpatterns = [
    url(
        r'^enterprise_course_enrollments/$',
        EnterpriseCourseEnrollmentView.as_view(),
        name="enterprise-learner-portal-course-enrollment-list"
    ),
]



"""
Management command for assigning enterprise_learner roles
to existing linked enterprise users that are missing them.
"""

import logging
from time import sleep

from django.contrib import auth
from django.core.exceptions import MultipleObjectsReturned
from django.core.management.base import BaseCommand

from enterprise.constants import ENTERPRISE_LEARNER_ROLE
from enterprise.models import EnterpriseCustomerUser, SystemWideEnterpriseRole, SystemWideEnterpriseUserRoleAssignment
from enterprise.utils import batch

log = logging.getLogger(__name__)
User = auth.get_user_model()


class Command(BaseCommand):
    """
    Management command for assigning enterprise_learner roles to existing enterprise users.

    Example usage:
        $ ./manage.py backfill_learner_role_assignments
    """
    help = 'Assigns enterprise_learner role to linked enterprise users missing them.'

    def add_arguments(self, parser):
        """
        Entry point for subclassed commands to add custom arguments.
        """
        parser.add_argument(
            '--batch-limit',
            action='store',
            dest='batch_limit',
            default=250,
            help='Number of records in each batch.',
            type=int,
        )

        parser.add_argument(
            '--batch-sleep',
            action='store',
            dest='batch_sleep',
            default=5,
            help='How long to sleep between batches.',
            type=int,
        )

    def backfill_learner_role_assignments(self, options):
        """
        Assigns enterprise_learner role to users.
        """
        batch_limit = options['batch_limit']
        batch_sleep = options['batch_sleep']

        role = SystemWideEnterpriseRole.objects.get(name=ENTERPRISE_LEARNER_ROLE)
        ecus = EnterpriseCustomerUser.objects.select_related('enterprise_customer').filter(linked=True)

        for ecu_batch in batch(ecus, batch_size=batch_limit):
            for ecu in ecu_batch:
                log.info('Processing EnterpriseCustomerUser %s', ecu)

                user = User.objects.get(id=ecu.user_id)
                enterprise_customer = ecu.enterprise_customer

                try:
                    role_assignment, created = SystemWideEnterpriseUserRoleAssignment.objects.get_or_create(
                        user=user,
                        role=role,
                        enterprise_customer=enterprise_customer
                    )
                except MultipleObjectsReturned:
                    log.info('Found MultipleObjectsReturned for user %s. Skipping.', user)
                    continue

                if created:
                    log.info(
                        'Created SystemWideEnterpriseUserRoleAssignment %s for enterprise user %s',
                        role_assignment, ecu
                    )
                else:
                    log.info(
                        'Did not create role assignment for enterprise user %s', ecu
                    )

            sleep(batch_sleep)

    def handle(self, *args, **options):
        """
        Entry point for management command execution.
        """
        log.info('Starting assigning enterprise_learner roles to users!')

        self.backfill_learner_role_assignments(options)

        log.info('Successfully finished assigning enterprise_learner roles to users!')



# Generated by Django 2.2.24 on 2021-10-05 10:52

import jsonfield.fields

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0004_contentmetadataitemtransmission_enterprise_customer_catalog_uuid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contentmetadataitemtransmission',
            name='channel_metadata',
            field=jsonfield.fields.JSONField(),
        ),
    ]



"""
Generic content metadata transmitter for integrated channels.
"""

import json
import logging
from datetime import datetime
from itertools import islice

from django.apps import apps
from django.conf import settings

from integrated_channels.exceptions import ClientError
from integrated_channels.integrated_channel.client import IntegratedChannelApiClient
from integrated_channels.integrated_channel.transmitters import Transmitter
from integrated_channels.utils import chunks, generate_formatted_log

LOGGER = logging.getLogger(__name__)


class ContentMetadataTransmitter(Transmitter):
    """
    Used to transmit content metadata to an integrated channel.
    """

    def __init__(self, enterprise_configuration, client=IntegratedChannelApiClient):
        """
        By default, use the abstract integrated channel API client which raises an error when used if not subclassed.
        """
        super().__init__(
            enterprise_configuration=enterprise_configuration,
            client=client
        )

    def _log_info(self, msg, content_id=None):
        LOGGER.info(
            generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                self.enterprise_configuration.enterprise_customer.uuid,
                None,
                content_id,
                msg
            )
        )

    def _log_error(self, msg):
        LOGGER.error(
            generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                self.enterprise_configuration.enterprise_customer.uuid,
                None,
                None,
                msg
            )
        )

    def transmit(self, create_payload, update_payload, delete_payload, content_updated_mapping, **kwargs):
        """
        Transmit content metadata items to the integrated channel. Save or update content metadata records according to
        the type of transmission.
        """
        self._log_info(
            f'Transmitting delete payload: {delete_payload} for customer: '
            f'{self.enterprise_configuration.enterprise_customer.uuid}'
        )
        self._transmit_delete(delete_payload)

        self._log_info(
            f'Transmitting create payload: {create_payload} for customer: '
            f'{self.enterprise_configuration.enterprise_customer.uuid}'
        )
        self._transmit_create(create_payload, content_updated_mapping)

        self._log_info(
            f'Transmitting update payload: {update_payload} for customer: '
            f'{self.enterprise_configuration.enterprise_customer.uuid}'
        )
        self._transmit_update(update_payload)

    def _prepare_items_for_transmission(self, channel_metadata_items):
        """
        Perform any necessary modifications to content metadata item
        data structure before transmission. This can be overridden by
        subclasses to add any data structure wrappers expected by the
        integrated channel.
        """
        return channel_metadata_items

    def _serialize_items(self, channel_metadata_items):
        """
        Serialize content metadata items for a create transmission to the integrated channel.
        """
        return json.dumps(
            self._prepare_items_for_transmission(channel_metadata_items),
            sort_keys=True
        ).encode('utf-8')

    def _transmit_create(self, channel_metadata_item_map, content_updated_mapping):
        """
        Transmit content metadata creation to integrated channel.
        """
        chunk_items = chunks(channel_metadata_item_map, self.enterprise_configuration.transmission_chunk_size)
        transmission_limit = settings.INTEGRATED_CHANNELS_API_CHUNK_TRANSMISSION_LIMIT.get(
            self.enterprise_configuration.channel_code()
        )
        for chunk in islice(chunk_items, transmission_limit):
            serialized_chunk = self._serialize_items(list(chunk.values()))
            try:
                self.client.create_content_metadata(serialized_chunk)
            except ClientError as exc:
                self._log_error(
                    f"Failed to create [{len(chunk)}] content metadata items for integrated channel "
                    f"[{self.enterprise_configuration.enterprise_customer.name}] "
                    f"[{self.enterprise_configuration.channel_code()}]. "
                    f"Task failed with message [{exc.message}] and status code [{exc.status_code}]"
                )
                LOGGER.exception(exc)
            else:
                self._create_transmissions(chunk, content_updated_mapping)

    def _transmit_update(self, update_payload):
        """
        Transmit content metadata update to integrated channel.
        """
        updated_metadata_mapping = {key: item.channel_metadata for key, item in update_payload.items()}
        chunk_items = chunks(updated_metadata_mapping, self.enterprise_configuration.transmission_chunk_size)
        transmission_limit = settings.INTEGRATED_CHANNELS_API_CHUNK_TRANSMISSION_LIMIT.get(
            self.enterprise_configuration.channel_code()
        )
        for chunk in islice(chunk_items, transmission_limit):
            serialized_chunk = self._serialize_items(list(chunk.values()))
            try:
                self.client.update_content_metadata(serialized_chunk)
            except ClientError as exc:
                self._log_error(
                    f"Failed to update [{len(chunk)}] content metadata items for integrated channel "
                    f"[{self.enterprise_configuration.enterprise_customer.name}] "
                    f"[{self.enterprise_configuration.channel_code()}]. "
                    f"Task failed with message [{exc.message}] and status code [{exc.status_code}]"
                )
                LOGGER.exception(exc)
            else:
                self._update_transmissions(update_payload, chunk)

    def _transmit_delete(self, channel_metadata_item_map):
        """
        Transmit content metadata deletion to integrated channel.
        """
        items_to_delete = {key: item.channel_metadata for key, item in channel_metadata_item_map.items()}
        chunk_items = chunks(items_to_delete, self.enterprise_configuration.transmission_chunk_size)
        transmission_limit = settings.INTEGRATED_CHANNELS_API_CHUNK_TRANSMISSION_LIMIT.get(
            self.enterprise_configuration.channel_code()
        )
        for chunk in islice(chunk_items, transmission_limit):
            serialized_chunk = self._serialize_items(list(chunk.values()))
            try:
                self.client.delete_content_metadata(serialized_chunk)
            except ClientError as exc:
                self._log_error(
                    f"Failed to delete [{len(chunk)}] content metadata items for integrated channel "
                    f"[{self.enterprise_configuration.enterprise_customer.name}] "
                    f"[{self.enterprise_configuration.channel_code()}]. "
                    f"Task failed with message [{exc.message}] and status code [{exc.status_code}]"
                )
                LOGGER.exception(exc)
            else:
                self._delete_transmissions(chunk.keys())

    def _create_transmissions(self, content_metadata_item_map, content_updated_mapping):
        """
        Create ContentMetadataItemTransmission models for the given content metadata items. Because records are soft
        deleted, before creating new records we must verify that there is no previously deleted record under this
        customer with this content ID. Additionally we must also check if a record exists under this customer under a
        separate catalog as content over catalogs is overwritten.
        """
        ContentMetadataItemTransmission = apps.get_model(
            'integrated_channel',
            'ContentMetadataItemTransmission'
        )
        create_transmissions = []
        for content_id, channel_metadata in content_metadata_item_map.items():
            catalog_uuid = content_updated_mapping.get(content_id).get('catalog_uuid')
            content_last_changed = content_updated_mapping.get(content_id).get('modified')
            self._log_info(
                f'Creating content transmission record for course: {content_id} under enterprise customer: '
                f'{self.enterprise_configuration.enterprise_customer.uuid}.',
                content_id=content_id
            )
            past_deleted_transmission = ContentMetadataItemTransmission.objects.filter(
                enterprise_customer=self.enterprise_configuration.enterprise_customer,
                integrated_channel_code=self.enterprise_configuration.channel_code(),
                content_id=content_id,
                deleted_at__isnull=False,
            ).first()

            if past_deleted_transmission:
                self._log_info(
                    f'Found previously deleted content record while creating record for course: {content_id}'
                    f'under customer: {self.enterprise_configuration.enterprise_customer.uuid}. Marking record as '
                    f'active.',
                    content_id=content_id
                )
                past_deleted_transmission.deleted_at = None
                past_deleted_transmission.channel_metadata = channel_metadata
                past_deleted_transmission.content_last_changed = content_last_changed
                past_deleted_transmission.enterprise_customer_catalog_uuid = catalog_uuid
                past_deleted_transmission.save()
            else:
                # Does there exist a record under a different catalog uuid?
                past_transmission = ContentMetadataItemTransmission.objects.filter(
                    enterprise_customer=self.enterprise_configuration.enterprise_customer,
                    integrated_channel_code=self.enterprise_configuration.channel_code(),
                    content_id=content_id,
                    deleted_at__isnull=True,
                ).first()
                if past_transmission:
                    self._log_info(
                        f'Found past content record under another catalog while creating record for course: '
                        f'{content_id} under customer: {self.enterprise_configuration.enterprise_customer.uuid}. '
                        f'Updating records customer catalog uuid to {catalog_uuid}.',
                        content_id=content_id
                    )
                    past_transmission.channel_metadata = channel_metadata
                    past_transmission.content_last_changed = content_last_changed
                    past_transmission.enterprise_customer_catalog_uuid = catalog_uuid
                    past_transmission.save()
                else:
                    create_transmissions.append(
                        ContentMetadataItemTransmission(
                            enterprise_customer=self.enterprise_configuration.enterprise_customer,
                            integrated_channel_code=self.enterprise_configuration.channel_code(),
                            content_id=content_id,
                            channel_metadata=channel_metadata,
                            content_last_changed=content_last_changed,
                            enterprise_customer_catalog_uuid=catalog_uuid
                        )
                    )
        ContentMetadataItemTransmission.objects.bulk_create(create_transmissions, batch_size=50, ignore_conflicts=True)

    def _update_transmissions(self, update_payload, content_metadata_item_map):
        """
        Update ContentMetadataItemTransmission models for the given content metadata items.
        """
        for content_id in content_metadata_item_map.keys():
            transmission = update_payload.get(content_id)
            if transmission:
                self._log_info(
                    f'Updating content transmission record for course: {content_id} under enterprise customer: '
                    f'{self.enterprise_configuration.enterprise_customer.uuid}.',
                    content_id=content_id
                )
                transmission.save()

    def _delete_transmissions(self, content_metadata_item_ids):
        """
        Delete ContentMetadataItemTransmission models associated with the given content metadata items.
        """
        ContentMetadataItemTransmission = apps.get_model(
            'integrated_channel',
            'ContentMetadataItemTransmission'
        )
        self._log_info(
            f'Marking content transmission record for courses: {content_metadata_item_ids} as deleted for customer: '
            f'{self.enterprise_configuration.enterprise_customer.uuid}'
        )
        ContentMetadataItemTransmission.objects.filter(
            enterprise_customer=self.enterprise_configuration.enterprise_customer,
            integrated_channel_code=self.enterprise_configuration.channel_code(),
            content_id__in=content_metadata_item_ids,
        ).update(deleted_at=datetime.now())



"""
    url mappings for integrated_channels/v1/sap_success_factors/
"""

from rest_framework import routers

from .views import SAPSuccessFactorsConfigurationViewSet

app_name = 'sap_success_factors'
router = routers.DefaultRouter()
router.register(r'configuration', SAPSuccessFactorsConfigurationViewSet, basename="configuration")
urlpatterns = router.urls



# Generated by Django 2.2.24 on 2021-08-02 22:45

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('enterprise', '0137_enrollment_email_update'),
    ]

    operations = [
        migrations.CreateModel(
            name='BulkCatalogQueryUpdateCommandConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('change_date', models.DateTimeField(auto_now_add=True, verbose_name='Change date')),
                ('enabled', models.BooleanField(default=False, verbose_name='Enabled')),
                ('arguments', models.TextField(blank=True, default='', help_text="Arguments for the 'bulk_update_catalog_query_id' management command. Specify like '<old ID> <new ID>'")),
                ('changed_by', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='Changed by')),
            ],
            options={
                'verbose_name': 'bulk_update_catalog_query_id argument',
            },
        ),
    ]



"""
Django admin integration for enterprise app.
"""

import json
import logging
from urllib.parse import urlencode

from config_models.admin import ConfigurationModelAdmin
from django_object_actions import DjangoObjectActions
from edx_rbac.admin import UserRoleAssignmentAdmin
from simple_history.admin import SimpleHistoryAdmin

from django.conf import settings
from django.conf.urls import url
from django.contrib import admin, auth
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.core.paginator import Paginator
from django.db import connection
from django.db.models import Q
from django.http import HttpResponseRedirect
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

from enterprise import constants
from enterprise.admin.actions import export_as_csv_action, refresh_catalog
from enterprise.admin.forms import (
    AdminNotificationForm,
    EnterpriseCustomerAdminForm,
    EnterpriseCustomerCatalogAdminForm,
    EnterpriseCustomerIdentityProviderAdminForm,
    EnterpriseCustomerReportingConfigAdminForm,
    EnterpriseFeatureUserRoleAssignmentForm,
    SystemWideEnterpriseUserRoleAssignmentForm,
)
from enterprise.admin.utils import UrlNames
from enterprise.admin.views import (
    EnterpriseCustomerManageLearnerDataSharingConsentView,
    EnterpriseCustomerManageLearnersView,
    EnterpriseCustomerTransmitCoursesView,
    TemplatePreviewView,
)
from enterprise.api_client.lms import CourseApiClient, EnrollmentApiClient
from enterprise.config.models import UpdateRoleAssignmentsWithCustomersConfig
from enterprise.models import (
    AdminNotification,
    AdminNotificationFilter,
    AdminNotificationRead,
    EnrollmentNotificationEmailTemplate,
    EnterpriseCatalogQuery,
    EnterpriseCourseEnrollment,
    EnterpriseCustomer,
    EnterpriseCustomerBrandingConfiguration,
    EnterpriseCustomerCatalog,
    EnterpriseCustomerIdentityProvider,
    EnterpriseCustomerInviteKey,
    EnterpriseCustomerReportingConfiguration,
    EnterpriseCustomerType,
    EnterpriseCustomerUser,
    EnterpriseFeatureUserRoleAssignment,
    PendingEnrollment,
    PendingEnterpriseCustomerAdminUser,
    PendingEnterpriseCustomerUser,
    SystemWideEnterpriseUserRoleAssignment,
)
from enterprise.utils import discovery_query_url, get_all_field_names, get_default_catalog_content_filter

try:
    from enterprise.api_client.enterprise_catalog import EnterpriseCatalogApiClient
except ImportError:
    EnterpriseCatalogApiClient = None

try:
    from openedx.features.enterprise_support.admin.views import EnrollmentAttributeOverrideView
except ImportError:
    EnrollmentAttributeOverrideView = None
User = auth.get_user_model()


logger = logging.getLogger(__name__)


class EnterpriseCustomerBrandingConfigurationInline(admin.StackedInline):
    """
    Django admin model for EnterpriseCustomerBrandingConfiguration.

    The admin interface has the ability to edit models on the same page as a parent model. These are called inlines.
    https://docs.djangoproject.com/en/1.8/ref/contrib/admin/#django.contrib.admin.StackedInline
    """

    model = EnterpriseCustomerBrandingConfiguration
    can_delete = False


class EnterpriseCustomerIdentityProviderInline(admin.StackedInline):
    """
    Django admin model for EnterpriseCustomerIdentityProvider.

    The admin interface has the ability to edit models on the same page as a parent model. These are called inlines.
    https://docs.djangoproject.com/en/1.8/ref/contrib/admin/#django.contrib.admin.StackedInline
    """

    model = EnterpriseCustomerIdentityProvider
    form = EnterpriseCustomerIdentityProviderAdminForm
    extra = 0


class EnterpriseCustomerCatalogInline(admin.TabularInline):
    """
    Django admin model for EnterpriseCustomerCatalog.
    The admin interface has the ability to edit models on the same page as a parent model. These are called inlines.
    https://docs.djangoproject.com/en/1.8/ref/contrib/admin/#django.contrib.admin.StackedInline
    """

    model = EnterpriseCustomerCatalog
    form = EnterpriseCustomerCatalogAdminForm
    extra = 0
    can_delete = False

    def get_formset(self, request, obj=None, **kwargs):
        formset = super().get_formset(request, obj, **kwargs)
        formset.form.base_fields['content_filter'].initial = get_default_catalog_content_filter()
        return formset


class PendingEnterpriseCustomerAdminUserInline(admin.TabularInline):
    """
    Django admin inline model for PendingEnterpriseCustomerAdminUser.
    """

    model = PendingEnterpriseCustomerAdminUser
    extra = 0
    fieldsets = (
        (None, {
            'fields': ('user_email', 'get_admin_registration_url')
        }),
    )
    readonly_fields = (
        'get_admin_registration_url',
    )

    def get_admin_registration_url(self, obj):
        """
        Formats the ``admin_registration_url`` model property as an HTML link.
        """
        return format_html('<a href="{0}">{0}</a>'.format(obj.admin_registration_url))

    get_admin_registration_url.short_description = 'Admin Registration Link'


@admin.register(EnterpriseCustomerType)
class EnterpriseCustomerTypeAdmin(admin.ModelAdmin):
    """
    Django admin model for EnterpriseCustomerType.
    """

    class Meta:
        model = EnterpriseCustomerType

    fields = (
        'name',
    )

    list_display = ('name', )
    search_fields = ('name', )


@admin.register(EnterpriseCustomer)
class EnterpriseCustomerAdmin(DjangoObjectActions, SimpleHistoryAdmin):
    """
    Django admin model for EnterpriseCustomer.
    """
    list_display = (
        'name',
        'slug',
        'customer_type',
        'site',
        'country',
        'active',
        'has_logo',
        'enable_dsc',
        'has_identity_provider',
        'uuid',
    )

    list_filter = ('active',)
    ordering = ('name',)
    search_fields = ('name', 'uuid',)
    inlines = [
        EnterpriseCustomerBrandingConfigurationInline,
        EnterpriseCustomerIdentityProviderInline,
        EnterpriseCustomerCatalogInline,
        PendingEnterpriseCustomerAdminUserInline,
    ]

    EXPORT_AS_CSV_FIELDS = ['name', 'active', 'site', 'uuid', 'identity_provider']

    actions = [
        export_as_csv_action('CSV Export', fields=EXPORT_AS_CSV_FIELDS),
    ]

    change_actions = ('manage_learners', 'manage_learners_data_sharing_consent', 'transmit_courses_metadata')

    form = EnterpriseCustomerAdminForm

    class Meta:
        model = EnterpriseCustomer

    def get_search_results(self, request, queryset, search_term):
        original_queryset = queryset
        queryset, use_distinct = super().get_search_results(request, queryset, search_term)

        # default admin search uses `__icontains` and it will return nothing if uuid
        # contains hyphens even if that uuid has an associated record present in model
        # for more details see https://code.djangoproject.com/ticket/29915
        if not queryset:
            queryset, use_distinct = super().get_search_results(
                request, original_queryset, search_term.replace('-', '')
            )

        return queryset, use_distinct

    def change_view(self, request, object_id, form_url='', extra_context=None):
        catalog_uuid = EnterpriseCustomerCatalogAdminForm.get_catalog_preview_uuid(request.POST)
        if catalog_uuid:
            catalog_content_metadata_url = \
                EnterpriseCatalogApiClient.get_content_metadata_url(catalog_uuid)
            return HttpResponseRedirect(catalog_content_metadata_url)
        return super().change_view(
            request,
            object_id,
            form_url,
            extra_context=extra_context
        )

    def get_form(self, request, obj=None, change=False, **kwargs):
        """
        Retrieve the appropriate form to use, saving the request user
        into the form for use in loading catalog details
        """
        form = super().get_form(request, obj, change, **kwargs)
        form.user = request.user
        return form

    def enable_dsc(self, instance):
        """
        Return True if data sharing consent is enabled for EnterpriseCustomer.

        Arguments:
            instance (enterprise.models.EnterpriseCustomer): `EnterpriseCustomer` model instance
        """
        return instance.enable_data_sharing_consent

    enable_dsc.boolean = True
    enable_dsc.short_description = 'Enable DSC'

    def has_logo(self, instance):
        """
        Return True if EnterpriseCustomer has a logo.

        Arguments:
            instance (enterprise.models.EnterpriseCustomer): `EnterpriseCustomer` model instance
        """
        has_logo = False
        if hasattr(instance, 'branding_configuration') and instance.branding_configuration.logo:
            has_logo = True

        return has_logo

    has_logo.boolean = True
    has_logo.short_description = 'Logo'

    def has_identity_provider(self, instance):
        """
        Return True if EnterpriseCustomer has related identity provider.

        Arguments:
            instance (enterprise.models.EnterpriseCustomer): `EnterpriseCustomer` model instance
        """
        return hasattr(instance, 'enterprise_customer_identity_providers')

    has_identity_provider.boolean = True
    has_identity_provider.short_description = 'Identity provider'

    def manage_learners_data_sharing_consent(self, request, obj):
        """
        Object tool handler method - redirects to "Clear Learners Data Sharing Consent" view
        """
        # url names coming from get_urls are prefixed with 'admin' namespace
        return HttpResponseRedirect(reverse("admin:" + UrlNames.MANAGE_LEARNERS_DSC, args=(obj.uuid,)))

    manage_learners_data_sharing_consent.label = "Clear Data Sharing Consent"
    manage_learners_data_sharing_consent.short_description = "Clear Data Sharing Consent for a Learner."

    def manage_learners(self, request, obj):
        """
        Object tool handler method - redirects to "Manage Learners" view
        """
        # url names coming from get_urls are prefixed with 'admin' namespace
        manage_learners_url = reverse("admin:" + UrlNames.MANAGE_LEARNERS, args=(obj.uuid,))
        return HttpResponseRedirect(manage_learners_url)

    manage_learners.label = "Manage Learners"
    manage_learners.short_description = "Allows managing learners for this Enterprise Customer"

    def transmit_courses_metadata(self, request, obj):
        """
        Object tool handler method - redirects to `Transmit Courses Metadata` view.
        """
        # url names coming from get_urls are prefixed with 'admin' namespace
        transmit_courses_metadata_url = reverse('admin:' + UrlNames.TRANSMIT_COURSES_METADATA, args=(obj.uuid,))
        return HttpResponseRedirect(transmit_courses_metadata_url)

    transmit_courses_metadata.label = 'Transmit Courses Metadata'
    transmit_courses_metadata.short_description = 'Transmit courses metadata for this Enterprise Customer'

    def get_urls(self):
        """
        Returns the additional urls used by the custom object tools.
        """
        customer_urls = [
            url(
                r"^([^/]+)/manage_learners$",
                self.admin_site.admin_view(EnterpriseCustomerManageLearnersView.as_view()),
                name=UrlNames.MANAGE_LEARNERS
            ),
            url(
                r"^([^/]+)/clear_learners_data_sharing_consent",
                self.admin_site.admin_view(EnterpriseCustomerManageLearnerDataSharingConsentView.as_view()),
                name=UrlNames.MANAGE_LEARNERS_DSC
            ),
            url(
                r"^([^/]+)/transmit_courses_metadata",
                self.admin_site.admin_view(EnterpriseCustomerTransmitCoursesView.as_view()),
                name=UrlNames.TRANSMIT_COURSES_METADATA
            )
        ]
        return customer_urls + super().get_urls()


@admin.register(EnterpriseCustomerUser)
class EnterpriseCustomerUserAdmin(admin.ModelAdmin):
    """
    Django admin model for EnterpriseCustomerUser.
    """

    class Meta:
        model = EnterpriseCustomerUser

    fields = (
        'user_id',
        'enterprise_customer',
        'user_email',
        'username',
        'created',
        'enterprise_enrollments',
        'other_enrollments',
        'invite_key'
    )

    # Only include fields that are not database-backed; DB-backed fields
    # are dynamically set as read only.
    readonly_fields = (
        'user_email',
        'username',
        'created',
        'enterprise_enrollments',
        'other_enrollments',
    )

    list_display = ('username', 'user_email', 'get_enterprise_customer')
    search_fields = ('user_id',)

    def get_enterprise_customer(self, obj):
        """
        Returns the name of enterprise customer linked with the enterprise customer user.
        """
        return obj.enterprise_customer.name
    get_enterprise_customer.short_description = 'Enterprise Customer'

    def get_search_results(self, request, queryset, search_term):
        search_term = search_term.strip()
        use_distinct = False

        if search_term:
            queryset = EnterpriseCustomerUser.objects.filter(
                user_id__in=User.objects.filter(
                    Q(email__icontains=search_term) | Q(username__icontains=search_term)
                )
            )
        else:
            queryset, use_distinct = super().get_search_results(
                request,
                queryset,
                search_term
            )

        return queryset, use_distinct

    def enterprise_enrollments(self, enterprise_customer_user):
        """
        Return a string representing a given EnterpriseCustomerUser's enterprise course enrollments

        Args:
            enterprise_customer_user: The instance of EnterpriseCustomerUser
                being rendered with this admin form.
        """
        enterprise_course_ids = self._get_enterprise_course_enrollments(enterprise_customer_user)
        courses_string = mark_safe(self.get_enrolled_course_string(enterprise_course_ids))
        return courses_string or 'None'

    def other_enrollments(self, enterprise_customer_user):
        """
        Return a string representing a given EnterpriseCustomerUser's non-enterprise course enrollments

        Args:
            enterprise_customer_user: The instance of EnterpriseCustomerUser
                being rendered with this admin form.
        """
        all_course_ids = self._get_all_enrollments(enterprise_customer_user)
        enterprise_course_ids = self._get_enterprise_course_enrollments(enterprise_customer_user)
        # remove overlapping enterprise enrollments from all enrollments
        course_ids = set(all_course_ids) - set(enterprise_course_ids)
        courses_string = mark_safe(self.get_enrolled_course_string(course_ids))
        return courses_string or 'None'

    def _get_enterprise_course_enrollments(self, enterprise_customer_user):
        """
        Return a list of course ids representing a given EnterpriseCustomerUser's enterprise course enrollments

        Args:
            enterprise_customer_user: The instance of EnterpriseCustomerUser
                being rendered with this admin form.
        """
        enrollments = EnterpriseCourseEnrollment.objects.filter(enterprise_customer_user=enterprise_customer_user)
        return [enrollment.course_id for enrollment in enrollments]

    def _get_all_enrollments(self, enterprise_customer_user):
        """
        Return a list of course ids representing a given EnterpriseCustomerUser's course enrollments,
        including both enterprise and non-enterprise course enrollments

        Args:
            enterprise_customer_user: The instance of EnterpriseCustomerUser
                being rendered with this admin form.
        """
        enrollment_client = EnrollmentApiClient()
        enrollments = enrollment_client.get_enrolled_courses(enterprise_customer_user.username)
        return [enrollment['course_details']['course_id'] for enrollment in enrollments]

    def get_readonly_fields(self, request, obj=None):
        """
        Make all fields readonly when editing existing model.
        """
        readonly_fields = super().get_readonly_fields(request, obj=obj)
        if obj:  # editing an existing object
            return readonly_fields + tuple(get_all_field_names(self.model))
        return readonly_fields

    def get_enrolled_course_string(self, course_ids):
        """
        Get an HTML string representing the courses the user is enrolled in.
        """
        courses_client = CourseApiClient()
        course_details = []
        for course_id in course_ids:
            name = courses_client.get_course_details(course_id)['name']
            course_details.append({'course_id': course_id, 'course_name': name})
        template = '<a href="{url}">{course_name}</a>'
        joiner = '<br/>'
        return joiner.join(
            template.format(
                url=reverse('about_course', args=[course['course_id']]),
                course_name=course['course_name'],
            )
            for course in course_details
        )


@admin.register(PendingEnterpriseCustomerUser)
class PendingEnterpriseCustomerUserAdmin(admin.ModelAdmin):
    """
    Django admin model for PendingEnterpriseCustomerUser
    """

    class Meta:
        model = PendingEnterpriseCustomerUser

    fields = (
        'user_email',
        'enterprise_customer',
        'created'
    )

    readonly_fields = (
        'user_email',
        'enterprise_customer',
        'created'
    )


@admin.register(PendingEnterpriseCustomerAdminUser)
class PendingEnterpriseCustomerAdminUserAdmin(admin.ModelAdmin):
    """
    Django admin model for PendingEnterpriseCustomerAdminUser
    """

    class Meta:
        model = PendingEnterpriseCustomerAdminUser

    fields = (
        'user_email',
        'enterprise_customer',
        'get_admin_registration_url',
    )

    readonly_fields = (
        'get_admin_registration_url',
    )

    list_display = (
        'user_email',
        'get_enterprise_customer',
        'get_admin_registration_url',
    )

    search_fields = (
        'user_email',
        'enterprise_customer__name',
    )

    def get_enterprise_customer(self, obj):
        """
        Returns the name of the associated EnterpriseCustomer.
        """
        return obj.enterprise_customer.name

    get_enterprise_customer.short_description = 'Enterprise Customer'

    def get_admin_registration_url(self, obj):
        """
        Formats the ``admin_registration_url`` model property as an HTML link.
        """
        return format_html('<a href="{0}">{0}</a>'.format(obj.admin_registration_url))

    get_admin_registration_url.short_description = 'Admin Registration Link'


@admin.register(EnrollmentNotificationEmailTemplate)
class EnrollmentNotificationEmailTemplateAdmin(DjangoObjectActions, admin.ModelAdmin):
    """
    Django admin for EnrollmentNotificationEmailTemplate model
    """
    change_actions = ("preview_as_course", "preview_as_program")

    class Meta:
        model = EnrollmentNotificationEmailTemplate

    def get_urls(self):
        """
        Returns the additional urls used by the custom object tools.
        """
        preview_urls = [
            url(
                r"^([^/]+)/preview/([a-z]+)/$",
                self.admin_site.admin_view(TemplatePreviewView.as_view()),
                name=UrlNames.PREVIEW_EMAIL_TEMPLATE
            )
        ]
        return preview_urls + super().get_urls()

    def preview(self, obj, preview_type):
        """
        Object tool handler method - redirects to "Preview" view
        """
        # url names coming from get_urls are prefixed with 'admin' namespace
        preview_url = reverse("admin:" + UrlNames.PREVIEW_EMAIL_TEMPLATE, args=(obj.pk, preview_type))
        return HttpResponseRedirect(preview_url)

    def preview_as_course(self, request, obj):
        """
        Redirect to preview the HTML template in the context of a course.
        """
        return self.preview(obj, 'course')

    preview_as_course.label = _("Preview (course)")
    preview_as_course.short_description = _(
        "Preview the HTML template rendered in the context of a course enrollment."
    )

    def preview_as_program(self, request, obj):
        """
        Redirect to preview the HTML template in the context of a program.
        """
        return self.preview(obj, 'program')

    preview_as_program.label = _("Preview (program)")
    preview_as_program.short_description = _(
        "Preview the HTML template rendered in the context of a program enrollment."
    )


@admin.register(EnterpriseCourseEnrollment)
class EnterpriseCourseEnrollmentAdmin(admin.ModelAdmin):
    """
    Django admin model for EnterpriseCourseEnrollment
    """

    class Meta:
        model = EnterpriseCourseEnrollment

    readonly_fields = (
        'enterprise_customer_user',
        'course_id',
        'saved_for_later',
        'license_uuid',
    )

    list_display = (
        'enterprise_customer_user',
        'course_id',
        'saved_for_later',
    )

    change_list_template = "enterprise/admin/enterprise_course_enrollments_list.html"
    search_fields = ('enterprise_customer_user__user_id', 'course_id',)

    def license_uuid(self, obj):
        """
        Return the subscription license UUID (if any exists)  associated with this enrollment.
        """
        return str(obj.license.license_uuid)

    def has_add_permission(self, request):
        """
        Disable add permission for EnterpriseCourseEnrollment.
        """
        return False

    def has_delete_permission(self, request, obj=None):
        """
        Disable deletion for EnterpriseCourseEnrollment.
        """
        features = getattr(settings, 'FEATURES', {})
        return features.get(constants.ALLOW_ADMIN_ENTERPRISE_COURSE_ENROLLMENT_DELETION, False)

    def changelist_view(self, request, extra_context=None):
        """
        Override to conditionally show the button.
        """
        extra_context = extra_context or {}
        extra_context['attr_override_button'] = bool(EnrollmentAttributeOverrideView)
        return super().changelist_view(request, extra_context=extra_context)

    def get_urls(self):
        """
        Append `Enrollment Attribute Override` view url with default urls
        """
        custom_urls = []
        if EnrollmentAttributeOverrideView:
            custom_urls = [
                url(
                    r'^override_attributes/$',
                    self.admin_site.admin_view(EnrollmentAttributeOverrideView.as_view()),
                    name='enterprise_override_attributes'
                ),
            ]

        return custom_urls + super().get_urls()


@admin.register(PendingEnrollment)
class PendingEnrollmentAdmin(admin.ModelAdmin):
    """
    Django admin model for PendingEnrollment
    """

    class Meta:
        model = PendingEnrollment

    readonly_fields = (
        'user',
        'course_id',
        'course_mode',
    )

    list_display = (
        'user',
        'course_id',
        'course_mode',
    )

    search_fields = ('user__user_email', 'course_id',)

    def has_add_permission(self, request):
        """
        Disable add permission for PendingEnrollment.
        """
        return False

    def has_delete_permission(self, request, obj=None):
        """
        Disable deletion for PendingEnrollment.
        """
        return False


@admin.register(EnterpriseCatalogQuery)
class EnterpriseCatalogQueryAdmin(admin.ModelAdmin):
    """
    Django admin model for EnterpriseCatalogQuery.
    """

    class Meta:
        model = EnterpriseCatalogQuery

    list_display = (
        'title',
        'discovery_query_url'
    )

    def discovery_query_url(self, obj):
        """
        Return discovery url for preview.
        """
        return discovery_query_url(obj.content_filter)

    def has_delete_permission(self, request, obj=None):
        return False

    readonly_fields = ('discovery_query_url', 'uuid')
    discovery_query_url.allow_tags = True
    discovery_query_url.short_description = 'Preview Catalog Courses'


@admin.register(EnterpriseCustomerCatalog)
class EnterpriseCustomerCatalogAdmin(admin.ModelAdmin):
    """
    Django admin model for EnterpriseCustomerCatalog.
    """
    ordering = ('enterprise_customer__name', 'title')
    actions = [refresh_catalog]

    class Meta:
        model = EnterpriseCustomerCatalog

    class Media:
        js = ('enterprise/admin/enterprise_customer_catalog.js',)

    list_display = (
        'uuid_nowrap',
        'enterprise_customer',
        'title',
        'preview_catalog_url',
    )

    search_fields = (
        'uuid',
        'title',
        'enterprise_customer__name',
        'enterprise_customer__uuid',
    )

    fields = (
        'title',
        'enterprise_customer',
        'enterprise_catalog_query',
        'content_filter',
        'enabled_course_modes',
        'publish_audit_enrollment_urls',
    )

    def preview_catalog_url(self, obj):
        """
        Return enterprise catalog url for preview.
        """
        catalog_content_metadata_url = \
            EnterpriseCatalogApiClient.get_content_metadata_url(obj.uuid)
        return format_html(
            '<a href="{url}" target="_blank">Preview</a>',
            url=catalog_content_metadata_url
        )

    readonly_fields = ('preview_catalog_url',)
    preview_catalog_url.short_description = 'Preview Catalog Courses'

    def uuid_nowrap(self, obj):
        """
        Inject html for disabling wrap for uuid
        """
        return format_html('<span style="white-space: nowrap;">{uuid}</span>'.format(uuid=obj.uuid))
    uuid_nowrap.short_description = 'UUID'

    def get_form(self, request, obj=None, change=False, **kwargs):
        form = super().get_form(request, obj, change, **kwargs)
        form.base_fields['content_filter'].initial = get_default_catalog_content_filter()
        return form

    def get_actions(self, request):
        """
        Disallow the delete selected action as that does not send a DELETE request to enterprise-catalog
        """
        actions = super().get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions


@admin.register(EnterpriseCustomerReportingConfiguration)
class EnterpriseCustomerReportingConfigurationAdmin(admin.ModelAdmin):
    """
    Django admin model for EnterpriseCustomerReportingConfiguration.
    """

    list_display = (
        "enterprise_customer",
        "active",
        "delivery_method",
        "frequency",
        "data_type",
        "report_type",
    )

    list_filter = ("active",)
    search_fields = ("enterprise_customer__name", "email")
    autocomplete_fields = ['enterprise_customer']
    ordering = ('enterprise_customer__name',)

    form = EnterpriseCustomerReportingConfigAdminForm

    class Meta:
        model = EnterpriseCustomerReportingConfiguration

    def get_fields(self, request, obj=None):
        """
        Return the fields that should be displayed on the admin form.
        """
        fields = list(super().get_fields(request, obj))
        if obj:
            # Exclude password fields when we are editing an existing model.
            return [f for f in fields if f not in {'decrypted_password', 'decrypted_sftp_password'}]

        return fields


class BigTableMysqlPaginator(Paginator):
    """
    A paginator that uses INFORMATION_SCHEMA.TABLES to estimate
    the total number of rows in a table.
    """
    ARBITRARILY_LARGE_NUMBER = 10000000

    # pylint: disable=attribute-defined-outside-init
    @property
    def count(self):  # pylint: disable=invalid-overridden-method
        """
        Returns the number of items in the object list (possibly an estimate).
        """
        query = self.object_list.query

        if query.where:
            return super().count
        if not getattr(self, '_whole_table_count', None):
            try:
                with connection.cursor() as cursor:
                    cursor.execute(
                        'SELECT TABLE_ROWS FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = %s',
                        [query.model._meta.db_table]
                    )
                    self._whole_table_count = int(cursor.fetchone()[0])
            except Exception:  # pylint: disable=broad-except
                logger.exception(
                    'Cannot get count estimate for %s from INFORMATION_SCHEMA.TABLES',
                    query.model,
                )
                self._whole_table_count = self.ARBITRARILY_LARGE_NUMBER
        return self._whole_table_count


@admin.register(SystemWideEnterpriseUserRoleAssignment)
class SystemWideEnterpriseUserRoleAssignmentAdmin(UserRoleAssignmentAdmin):
    """
    Django admin model for SystemWideEnterpriseUserRoleAssignment.
    """

    paginator = BigTableMysqlPaginator
    show_full_result_count = False

    fields = ('user', 'role', 'enterprise_customer', 'applies_to_all_contexts', 'effective_enterprise_customer')
    readonly_fields = ('effective_enterprise_customer',)

    list_display = ('user', 'role', 'enterprise_customer', 'applies_to_all_contexts')
    # This tells Django to use select_related() in retrieving the list of objects on the change list page.
    # It should save some queries
    list_select_related = (
        'user',
        'role',
        'enterprise_customer',
    )
    list_per_page = 25

    search_fields = ('user__email', 'role__name', 'enterprise_customer__name')

    form = SystemWideEnterpriseUserRoleAssignmentForm

    class Meta:
        model = SystemWideEnterpriseUserRoleAssignment

    def effective_enterprise_customer(self, instance):
        """
        Return the comma separated names of all enterprise customers attached to the user.

        Arguments:
            instance (SystemWideEnterpriseUserRoleAssignment): model instance
        """
        if instance.enterprise_customer:
            return instance.enterprise_customer

        enterprise_customers = EnterpriseCustomerUser.objects.filter(
            user_id=instance.user.id
        ).select_related(
            'enterprise_customer'
        ).values_list(
            'enterprise_customer__name',
            flat=True
        )
        if enterprise_customers.exists():
            return ', '.join(list(enterprise_customers))
        return None

    def get_form(self, request, obj=None, **kwargs):  # pylint: disable=arguments-differ
        """
        Adds help text to the callable-defined ``effective_enterprise_customer`` field.
        """
        kwargs.update({
            'help_texts': {
                'effective_enterprise_customer': (
                    'The comma-separated name(s) of all enterprise customers the user '
                    'is currently linked to (deprecated - we will eventually have explicitly defined '
                    'one enterprise customer per user role assignment via the Enterprise customer field).'
                ),
            },
        })
        return super().get_form(request, obj, **kwargs)


@admin.register(EnterpriseFeatureUserRoleAssignment)
class EnterpriseFeatureUserRoleAssignmentAdmin(UserRoleAssignmentAdmin):
    """
    Django admin model for EnterpriseFeatureUserRoleAssignment.
    """

    form = EnterpriseFeatureUserRoleAssignmentForm

    class Meta:
        model = EnterpriseFeatureUserRoleAssignment


admin.site.register(UpdateRoleAssignmentsWithCustomersConfig, ConfigurationModelAdmin)


@admin.register(AdminNotificationRead)
class AdminNotificationReadAdmin(admin.ModelAdmin):
    """
    Django admin for AdminNotificationRead model.
    """

    model = AdminNotificationRead
    list_display = ('id', 'enterprise_customer_user', 'admin_notification', 'is_read', 'created', 'modified')


@admin.register(AdminNotification)
class AdminNotificationAdmin(admin.ModelAdmin):
    """
    Django admin for AdminNotification model.
    """

    model = AdminNotification
    form = AdminNotificationForm
    list_display = ('id', 'title', 'text', 'is_active', 'start_date', 'expiration_date', 'created', 'modified')
    filter_horizontal = ('admin_notification_filter',)


@admin.register(AdminNotificationFilter)
class AdminNotificationFilterAdmin(admin.ModelAdmin):
    """
    Django admin for AdminNotificationFilter model.
    """

    model = AdminNotificationFilter
    list_display = ('id', 'filter', 'created', 'modified')


@admin.register(EnterpriseCustomerInviteKey)
class EnterpriseCustomerInviteKeyAdmin(admin.ModelAdmin):
    """
    Django admin model for EnterpriseCustomerInviteKey.
    """

    fields = (
        'enterprise_customer',
        'usage_count',
        'usage_limit',
        'expiration_date',
        'is_active',
    )

    readonly_fields = ('uuid', 'usage_count')

    list_display = (
        'uuid',
        'enterprise_customer_id',
        'usage_limit',
        'expiration_date',
        'is_active',
    )

    list_filter = ('is_active',)

    search_fields = (
        'uuid__startswith',
        'enterprise_customer__name__startswith',
    )

    class Meta:
        model = EnterpriseCustomerInviteKey

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super().get_readonly_fields(request, obj=obj)

        if not obj or not obj.is_active:
            return readonly_fields + ('is_active',)

        return readonly_fields



"""
URL definitions for edX Enterprise's Consent API endpoint.
"""

from django.conf.urls import include, url

urlpatterns = [
    url(
        r'^v1/',
        include('consent.api.v1.urls'),
        name='consent_api_v1'
    ),
]



# Generated by Django 2.2.17 on 2021-01-26 20:39

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0120_systemwiderole_applies_to_all_contexts'),
    ]

    operations = [
        migrations.AddField(
            model_name='systemwideenterpriseuserroleassignment',
            name='enterprise_customer',
            field=models.ForeignKey(blank=True, help_text='The EnterpriseCustomer for which the role is assigned for the user.  If creating a new assignment, you must first provide a user email and role, then click "Save and continue editing" BEFORE selecting from this dropdown.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='system_wide_role_assignments', to='enterprise.EnterpriseCustomer'),
        ),
    ]



from django.db import migrations

from enterprise.constants import USE_ENTERPRISE_CATALOG


def create_sample(apps, schema_editor):
    """Create the `use_enterprise_catalog` sample if it does not already exist."""
    Sample = apps.get_model('waffle', 'Sample')
    Sample.objects.get_or_create(
        name=USE_ENTERPRISE_CATALOG,
        defaults={'percent': 100},
    )


def delete_sample(apps, schema_editor):
    """Delete the `use_enterprise_catalog` sample if one exists."""
    Sample = apps.get_model('waffle', 'Sample')
    Sample.objects.filter(name=USE_ENTERPRISE_CATALOG).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0108_add_licensed_enrollment_is_revoked'),
    ]

    operations = [
        migrations.RunPython(delete_sample, create_sample)
    ]
